/* config.h.  Generated automatically by configure.  */
/* config.h.in.  Generated automatically from libAfterImage/configure.in by autoheader.  */

/* Define as __inline if that's what the C compiler calls it.  */
/* #undef inline */

/* Define if your processor stores words with the most significant
   byte first (like Motorola and SPARC, unlike Intel and VAX).  */
#define WORDS_BIGENDIAN 1

/* Define if the X Window System is missing or not being used.  */
/* #undef X_DISPLAY_MISSING */

/* #undef I18N */

/* debugging options */
/* #undef  DEBUG_ALLOCS */

#define  HAVE_MMX 1

#define  HAVE_XPM 1
/* #undef  HAVE_LIBXPM */
/* #undef  HAVE_LIBXPM_X11 */

#define  HAVE_JPEG 1
#define  HAVE_PNG 1
#define  HAVE_GIF 1
#define  HAVE_TIFF 1
#define  HAVE_FREETYPE 1
#define  HAVE_FREETYPE_FREETYPE 1

/* #undef  HAVE_AFTERBASE */

/* Define if you have the <ft2build.h> header file.  */
#define HAVE_FT2BUILD_H 1
